import { OpenEvent, CloseEvent } from './Events';
import { Offset } from './Offset';
import { PopupAnimation } from './PopupAnimation';
/**
 * @hidden
 */
export interface PopupSettings {
    /**
     * Controls the Popup animation ([see example]({% slug animations_popup %})). By default, the opening and closing animations are enabled.
     */
    animate?: boolean | PopupAnimation;
    /**
     * Specifies the element which will be used as an anchor ([see example]({% slug alignmentpositioning_popup %})). The Popup opens next to that element.
     */
    anchor: string;
    /**
     * Specifies the pivot point of the anchor ([see example]({% slug alignmentpositioning_popup %})).
     */
    anchorAlign?: object;
    /**
     * Defines the 'id' or 'ref' of the container to which the Popup will be appended. Defaults to [`body`](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body).
     */
    appendTo?: string;
    /**
     * Configures the collision behavior of the Popup ([see example]({% slug viewportboundarydetection_popup %})).
     */
    collision?: object;
    /**
     * Specifies the pivot point of the Popup ([see example]({% slug alignmentpositioning_popup %})).
     */
    popupAlign?: object;
    /**
     * Specifies a list of CSS classes that will be added to the Popup element.
     */
    className?: string | Array<string>;
    /**
     * Specifies the id that will be added to the Popup element.
     */
    id?: string;
    /**
     * Specifies the direction of the Expand Animation. Defaults to `down`.
     */
    direction?: String;
    /**
     * Specifies the direction of the Expand Animation. Defaults to `expand`.
     */
    transition?: String;
    /**
     * Specifies a list of CSS classes that will be added to the internal animated element ([see example]({% slug appearance_popup %})).
     */
    popupClass?: string | Array<string>;
    /**
     * Represents the styles that are applied to the Popup.
     */
    style?: any;
    /**
     * Specifies the absolute position of the element ([see example]({% slug alignmentpositioning_popup %})). The Popup opens next to that point. The pivot point of the Popup is defined by the `popupAlign` configuration option. The boundary detection is applied by using the window viewport.
     */
    offset?: Offset;
    /**
     * Fires after the Popup is opened and the opening animation ends.
     */
    onOpen?: (event: OpenEvent) => void;
    /**
     * Fires after the Popup is closed.
     */
    onClose?: (event: CloseEvent) => void;
}
